         LIST P=16C54
;	 
;        
;        Sample test program to power up serial EEPROM
;        using PIC port A, then write one byte and read
;        same byte, then repeat forever.
;
;*******************************************************
port_a  equ     5h     ; port 5 used for device
                       ; address select
port_b  equ     6h     ; port 6 used for data and 
                       ; clock lines
eeprom  equ    0ah     ; bit buffer
addr    equ    0ch     ; address register
datai   equ    0dh     ; stored data input reg.
datao   equ    0eh     ; stored data output reg.
slave   equ    0fh     ; device address
                       ; (1010xxx0)
txbuf   equ    10h     ; tx buffer
count   equ    11h     ; bit counter
bcount  equ    12h     ; byte counter
rxbuf   equ    13h     ; receive buffer
loops   equ    15h     ; delay loop counter
loops2  equ    16h     ; delay loop counter 2
;
;                     Bit Assignments
;
di      equ    7       ; eeprom input
do      equ    6       ; eeprom output
sdata   equ    7       ; data line (port_b)
sclk    equ    6       ; clock line (port_b)
vcc     equ    3       ; vcc for dut (port_a)
;
        org    01ffh
begin   goto   PWRUP
        org    000h
        goto   PWRUP  
;
;*********************************************************
;                DELAY ROUTINE
;                 this routine takes the value in loops
;                 and loops that many times.  Every increase
;                 in 'loops' yields approx 1 more millisecond.
;                 i.e., if 'loops' is 10 then the wait period
;                 is approx 10 milliseconds.
;     
;---------------------------------------------------------
WAIT
;

top2     movlw   .110
         movwf   loops2        
top      nop                    ; sit and wait
         nop
         nop
         nop
         nop
         nop
         decfsz  loops2         ; inner loop done?
         goto    top            ; no, go again
         decfsz  loops          ; outer loop done?
         goto    top2           ; no, go again
         retlw   0	        ; yes, return from sub
;
;*********************************************************
;                  Start Bit Subroutine
;                  this routine generates a start bit
;----------------------------------------------------------
;
BSTART
         movlw   b'00111111'
	 tris    port_b         ; port b for output
	 bsf     port_b,sdata   ; set clock high
         nop
         nop
         bsf     port_b,sclk    ; set clock high
         nop
         nop
         nop
         nop
         nop
         nop
         nop
         nop
	 bcf     port_b,sdata   ; data line goes low during
	                        ; high clock for start bit
	 nop
	 nop
         nop
         nop
         nop
         nop
	 bcf     port_b,sclk    ; start clock train
	 nop
         nop
         nop
         retlw   0
	 ;
	 ;    End of Subroutine
;************************************************************
;
;                 Stop Bit Subroutine
;                 this routine generates a stop bit
;---------------------------------------------------------------
BSTOP
         movlw   b'00111111'    ;
	 tris    port_b         ; set data/clock lines as outputs
         bcf     port_b,sdata   ; make sure data line is low
	 nop
         nop
         nop
         nop
	 nop
	 nop
	 bsf     port_b,sclk    ; set clock high
	 nop
         nop
         nop	
         nop
         nop
         nop
         bsf     port_b,sdata   ; data goes high while clock high
	                        ; for stop bit
	 nop
	 nop
         nop
         nop
         nop
         bcf     port_b,sclk    ; set clock low again
         nop
	 nop
         nop
         retlw   0
;
;           End of Subroutine
;*************************************************************
;       Serial data send 1 bit from PIC to dut
;-------------------------------------------------------------
BITOUT
         movlw   b'00111111'    ; set data,clock as outputs
	 tris    port_b
	 btfss   eeprom,do  
	 goto    BIT0
	 bsf     port_b,sdata   ; output bit 0
	 goto    CLK1           ; data line clocked low by device
;
BIT0
         bcf     port_b,sdata   ; output bit 0
CLK1     
         nop
         nop
	 bsf     port_b,sclk    ; set clock line high
BIT2
         nop
         nop
         nop
	 nop
	 bcf     port_b,sclk    ; return clock line low
	 retlw   0
;
;                   End of Subroutine
;	 	 	 	  	 			
;**************************************************************
;            Bit in routine
;               this routine gets a bit of data from the part
;               into the 'eeprom' register, bit 'di'
;--------------------------------------------------------------
BITIN
         movlw   b'10111111'    ; make sdata an input line
         tris    port_b
         bcf     eeprom,di      ; assume input bit low
         bsf     port_b,sclk    ; set clock line high
         nop                    ; just sit here a sec
         nop
         nop
         nop
         nop
         nop
         nop
         nop                    ;
         btfsc   port_b,sdata   ; read data line
         bsf     eeprom,di      ; set input bit if needed
         bcf     port_b,sclk    ; set clock line low
         retlw   0              ; hit the road
;
;**************************************************************
;  
;            Transmit Data Subroutine
;--------------------------------------------------------------
TX
         movlw   .8
         movwf   count          ; set the #bits to 8
         ;
TXLP
         bcf     eeprom,do
	 btfsc   txbuf,7
	 bsf     eeprom,do      ; otherwise data bit =1
	 call    BITOUT         ; serial data out
	 rlf     txbuf          ; rotate txbuf left
	 decfsz  count          ; 8 bits done?
	 goto    TXLP           ; no - go again
	 call    BITIN          ; read ack bit
	 ;
	 retlw   0
;                    End of Subroutine
;****************************************************************
;         Receive data Routine
;           this routine gets a byte of data from the part
;           into 'rxbuf'
;----------------------------------------------------------------
RX
         movlw   .8             ; set # bits to 8
         movwf   count      
         clrf    rxbuf          ; clear receive buffer
RXLP     rlf     rxbuf          ; rotate buffer left 1 bit
         bcf     rxbuf,0        ; assume bit is zero
         call    BITIN          ; read a bit
         btfsc   eeprom,di      ; input bit high?
         bsf     rxbuf,0        ; yes, set buffer bit high
         decfsz  count          ; 8 bits done?
         goto    RXLP           ; no, do another
         bcf     eeprom,do      ; set ack bit = 0
         call    BITOUT         ; to finish transmission
         retlw   0
;    
;****************************************************************
;         Power up routine
;           this routine blinks the lights
;----------------------------------------------------------------
PWRUP
          movlw  b'00000001'
          tris   port_a         ; set RA0 as input, rest output
          bsf    port_a,vcc     ; turn on power to dut         
          nop                   ; wait for dut to power up
          nop
          nop
          nop
          nop
;
;****************************************************************
;          Byte Write Routine
;          this writes the data in "55h" to the first byte
;          in the serial EEPROM.
;----------------------------------------------------------------
;
WRBYTE
                             ;
         movlw   b'10100000' ; set slave address and write mode
	 movwf   slave
	 movlw   b'01010101' ; set data to 55h
	 movwf   datao
                             ;	 
         clrf    addr        ; set address to 00h
                             ;
         call    BSTART      ; generate start bit
         movf    slave,w     ; get slave address
 	 movwf   txbuf       ; into transmit buffer
	 call    TX          ; and send it        
         movf    addr,w      ; get word address
         movwf   txbuf       ; into transmit buffer
         call    TX          ; and send it        
         movf    datao,w     ; move data
 	 movwf   txbuf       ; to tranmit buffer
 	 call    TX          ; and transmit it
         call    BSTOP       ; generate stop bit
                             ;
         movlw   .10 
         movwf   loops       ; set delay time to give
         call    WAIT        ; 10 ms wait after every byte
                             ;
                             ; now drop through and do the read
                             ;
;**********************************************************
;             READ (read routine)
;               this routine reads the first address 
;               of the dut
;----------------------------------------------------------
READ
                             ;
         movlw   b'10100000' ; set slave address and write mode
	 movwf   slave
                             ;	 
         clrf    addr        ; set address to 00h
                             ;
         call    BSTART      ; generate start bit
         nop
         nop
         movf    slave,w     ; get slave address
 	 movwf   txbuf       ; into transmit buffer
         call    TX          ; and send it        
         movf    addr,w      ; get word address
         movwf   txbuf       ; into transmit buffer
         call    TX          ; and send it        
         nop
         nop
         call    BSTART      ; generate start bit
         nop
         nop
         movlw   b'10100001' ; get slave address and read mode
         movwf   txbuf       ; into transmit buffer
         call    TX          ; and transmit it
         nop
         call    RX          ; get 8 bits of data
         bsf     eeprom,do
         call    BITOUT      ; send high ack bit and then a
         call    BSTOP       ; stop bit to end transmission from dut
         nop                 ; 
         nop
         nop
         nop
         nop
         bcf     port_a,vcc  ; turn power to dut off
         movlw   .100
         movwf   loops
         call    WAIT        ; wait awhile
         goto    PWRUP       ; go do the whole thing over again
;
END     	 
	 


