;*********************************************************************************************************
;				PutChar Function
;
;	Function to transmit A Byte Of Data
;	Before calling this routine, load the Byte to be transmitted into TxReg
;	Make sure _txmtProgress & _rcvOver bits (in Status Reg) are cleared before
;	calling this routine
;
;  Program Memory :	6 locations (10 locations if PARITY is Used)
;  Cycles	  :	8 (13 if PARITY is Used)
;
;*********************************************************************************************************
PutChar:
	bsf	_txmtEnable			; enable transmission
	bsf	_txmtProgress
	LOAD_BITCOUNT				; Macro to load bit count
;
   if _PARITY_ENABLE
   	movf	TxReg,W
	call	GenParity			; If Parity is used, then Generate Parity Bit
   endif  
;
	call	_TxmtStartBit
	bsf	_rtie				; Enable RTCC Overflow INT
	retfie					; return with _GIE Bit Set
;
;*********************************************************************************************************
;			Internal Subroutine
; entered from Interrupt Service Routine when Start Bit Is detected.
;
;  Program Memory :   15 locations  (25 locations if PARITY is used)
;  Cycles	  :   13 Worst Case
;
;*********************************************************************************************************

_TxmtNextBit:
	bcf	_rp0
	LOAD_RTCC  RtccPreLoad, RtccPrescale			; Macro to reload RTCC
;
  if _PARITY_ENABLE
	movf	BitCount
	btfsc	_z
	goto	_ParityOrStop
  endif
;
	decfsz	BitCount
	goto	_NextTxmtBit
;
  if _PARITY_ENABLE
_ParityOrStop:
	decfsz	ExtraBitCount
        goto	_SendParity
  endif
;
_StopBit:
	bsf	TX				; STOP Bit is High
	bcf	_rtie				; disable further interrupts
	bcf	_txmtProgress
	goto	RestoreIntStatus
;
_NextTxmtBit:
	rrf	TxReg
	btfss	_carry
        bcf	TX
	btfsc	_carry
	bsf	TX
;
	btfss	_txmtEnable
	bsf	_rtie				; disable further interrupts, Transmission Aborted	
;
	goto	RestoreIntStatus
;
  if _PARITY_ENABLE
_SendParity:
	btfss	_parityBit
	bcf	TX
	btfsc	_parityBit
        bsf	TX
	goto	RestoreIntStatus
  endif

;
;*********************************************************************************************************
;			Internal Subroutine
; entered from Interrupt Service Routine when Start Bit Is detected.
;
;  Program Memory :	9 locations
;  Cycles	  :	10
;
;*********************************************************************************************************
_TxmtStartBit:
	bsf	_rp0
	movlw	(_OPTION_INIT | RtccPrescale)
	movwf	_option				; Set Option Reg Located In Page 1
	bcf	_rp0				; make sure to select Page 0
	bcf	TX				; Send Start Bit
	movlw	-RtccPreLoad			; Prepare for Timing Interrupt
	movwf	_rtcc
	bcf	_rtif
	return

;*********************************************************************************************************
;				Generate Parity for the Value in WREG
;
;  The parity bit is set in _parityBit (SerialStatus,7)
;  Common Routine For Both Transmission & Reception
;
;  Program Memory :	13 locations
;  Cycles	  :	14
;
;*********************************************************************************************************
  if _PARITY_ENABLE

GenParity:
	movwf	temp2
	swapf	temp2,w
	xorwf	temp2,w
	movwf	temp1
        rrf	temp1
	rrf	temp1
	xorwf	temp1
	incf	temp1
	rrf	temp1		; parity bit in Bit 0
; Parity bit is in Bit 0 of temp1
;
    if _ODD_PARITY
	bsf	_parityBit
	btfsc	temp1,0
        bcf	_parityBit
    else
	bcf	_parityBit
	btfsc	temp1,0
        bsf	_parityBit
    endif

	return
  endif
;*********************************************************************************************************
